
# Graphics class
# Contains screen, sprite groups, etc
import pygame
import pygame.gfxdraw
import Util

from Actor import Actor
from AnimActor import AnimActor
from Player import Player
from Config import *
from pygame.locals import *
from Enemy import Enemy
import random
	
class Graphics():
	def __init__(self, game_obj):
		self.main = game_obj
		
		if IS_FULLSCREEN:
			self.screen = pygame.display.set_mode((WIDTH, HEIGHT), FULLSCREEN|DOUBLEBUF)
		else:
			self.screen = pygame.display.set_mode((WIDTH, HEIGHT))
		
		pygame.display.set_caption(GAME_TITLE)
	
		self.skull = []

		self.bgGroup = pygame.sprite.OrderedUpdates()
		self.playerGroup = pygame.sprite.RenderPlain()
		self.staticGroup = pygame.sprite.RenderPlain()
		#self.dynamicGroup = pygame.sprite.RenderPlain()	
		self.hudGroup = pygame.sprite.RenderPlain()

	def notifyStateChange(self):
		if self.main.state == "MENU":
			self.main.titleScreen.renew(self.screen)
		elif self.main.state == "PAUSE":
			self.main.pauseScreen.renew(self.screen)
		elif self.main.state == "PLAY":
			pass
		

	def loadHud(self):
		self.hud = Actor(HUD_IMAGE,(47,47,47))
		self.hud.rect.top = 0
		self.hud.rect.right = WIDTH
		self.hudGroup.add(self.hud)
		self.hudFont = pygame.font.Font(None, 30)

	def loadBackground(self, imagefile):
		# map object
		#tmap = self.main.theMap
		# Create background object
		self.bg = Actor(imagefile)
		# grab width/height in pixes of map
		#self.bgWidth = tmap.mapRows*TILE_SIZE
		#self.bgHeight = tmap.mapCols*TILE_SIZE
		#self.bg.setRect(Rect(0, 0, self.bgWidth, self.bgHeight))
		# dynamicLayer is an invisible layer on top of bg for all animated 
		# sprites to be drawn on
		#self.dynamicLayer = Actor()
		#self.dynamicLayer.setRect(self.bg.rect)
		#self.dynamicLayer.image.set_colorkey((255,255,255))
		# We must draw bg before dynamicLayer so use OrderedUpdates
		
		#self.dynamicGroup.add(self.dynamicLayer)
		self.bgGroup.add(self.bg)
		# position player in middle of map
		self.bg.rect.left = (HEIGHT/2 - self.bg.rect.width/2)
		self.bg.rect.top = HEIGHT/2 - self.bg.rect.height/2

#		# Load tiles
#		tset = tmap.getTileset()
#		x = self.bg.rect.left
#		y = self.bg.rect.top
#		for row in tmap.getMap():
#			for elem in row:
#				if elem in tset:
#					temp = Actor(tset[elem].strip())
#					temp.setPos((x + TILE_SIZE/2), (y + TILE_SIZE/2))
#					self.bgGroup.add(temp)
#				elif elem == 's':
#					self.startPos = (x, y)
#				x += TILE_SIZE
#			y += TILE_SIZE
#			x = self.bg.rect.left
#


	def loadPlayer(self):
		self.player = Player(playerAnimSpeed = 4)
		# Visible portion is 480x480, so "center" is (240, 240)
		self.player.setPos(240, HEIGHT/2)
		self.playerGroup.add(self.player)

	# This is temporary. We can't hardcode things like this
	def loadSkulls(self, imgfiles):
#		for i in range(0,9): # should be range(10)
#			self.skull[i] = Obstacle(imgfiles, -1)
#			self.skull[i].setStats(100,70,34,10)
#			self.skull[i].setPos(random.randint(30,600),random.randint(30,600))
#			self.staticGroup.add(self.skull[i])
		for i in range(9):
			self.skull.append(Enemy("skull.png", -1))
			self.skull[i].setPos(random.randint(240-self.bg.rect.width/2, 240+self.bg.rect.width/2),random.randint(240-self.bg.rect.height/2, 240+self.bg.rect.height/2))
			self.skull[i].setPos(random.randint(30,600),random.randint(30,600))
			self.staticGroup.add(self.skull[i])

	def updateGroups(self):
		self.bgGroup.update()
		self.staticGroup.update()
		#self.dynamicGroup.update()
		self.hudGroup.update()

		if self.main.animatePlayer:
			self.player.update()
		else:
			self.player.idle()

		for hit in pygame.sprite.spritecollide(self.player, self.staticGroup, \
			dokill = True):
			self.player.skullsLeft -= 1
			self.main.skullSound.play()

		# IMPORTANT: REMOVE NEWLINES BEFORE DRAWING TEXT
		# You probably want to use 1 for antialias value
		# Render has an optional "text background" that I do not implement
		# Apparently this can improve performance, especially w/ antialiased text
	def drawHUDText(self):
		# HUD fonts
		self.drawText("Skulls Left: " + str(self.player.skullsLeft), 1, \
			(255, 255, 255), self.hudFont, (WIDTH - HUD_WIDTH + 20, 20))

		self.drawText("Time Left: " + str(self.main.timeLeft), 1, \
			(255, 255, 255), self.hudFont, (WIDTH - HUD_WIDTH + 20, 40))


	def drawText(self, textMessage, textAntialias, textColor, textFont, textPosition):
			renderedText = textFont.render(textMessage, textAntialias, textColor)
			self.screen.blit(renderedText, textPosition)

	def drawScreen(self):
		if self.main.state == "PAUSE":
			if(self.main.pauseScreen.itemChanged()):
				self.main.pauseScreen.renderScreen(self.screen)
			
		
		elif self.main.state == "MENU":
			if(self.main.titleScreen.itemChanged()):
				self.main.titleScreen.renderScreen(self.screen)
					
		elif self.main.state == "PLAY":
			# Fill image layer with colorkey
			#self.dynamicLayer.image.fill((255,255,255))
			# draw background and skull layer
			
			self.bgGroup.draw(self.screen)
			self.staticGroup.draw(self.screen)
			# draw skulls onto layer

			#self.dynamicGroup.draw(self.dynamicLayer.image)
		
			#for i in self.staticGroup:
			#	pygame.gfxdraw.rectangle(self.screen, i.rect, (255,255,255))
			# draw player, hud, and fonts
			self.playerGroup.draw(self.screen)
			self.hudGroup.draw(self.screen)
			self.drawHUDText()
		
		pygame.display.flip()

