import pygame
import sys

from Actor import Actor
from Graphics import Graphics
from EventHandler import EventHandler
from Vector2 import Vector2
import Util
import data
from Config import *
from pygame.locals import *
from TitleScreen import TitleScreen
from PauseScreen import PauseScreen
# from Character import Character
from Obstacle import Obstacle

class Game():
	def __init__(self):
		pygame.init()
		
		#pygame.key.set_repeat(1,1)

		self.g = Graphics(self)
		
		
		self.keys = [] 	# List of directional keys pressed
		self.event = EventHandler(self)
		self.clock = pygame.time.Clock()
		
		self.timeLeft = 30
		self.counter = 0

		# TODO: reconsider instantiating these here
		self.titleScreen = TitleScreen()
		self.pauseScreen = PauseScreen() 
		
		# FSM to control game.
		self.state = "MENU"
		self.g.notifyStateChange()

		
		self.gameLoop()

	
	def gameLoop(self):
		while True:
			# Control frame rate
			self.clock.tick(FRAME_RATE)
			
			# Parse all events
			self.event.handleEvents()

			# Update game based on input
			self.update()

			# Draw all sprites on the screen
			self.g.drawScreen()

	def update(self):
		
		# Keep bg on screen
		# If attempting to move left and the left side of bg is on the edge
		if self.state == "PLAY":
			if self.bg.vel[0] > 0 and self.bg.rect.left >= 0:
				self.bg.vel[0] = 0
				#self.g.dynamicLayer.vel[0] = 0
			if self.bg.vel[0] < 0 and self.bg.rect.right <= WIDTH - HUD_WIDTH:
				self.bg.vel[0] = 0
				#self.g.dynamicLayer.vel[0] = 0
			if self.bg.vel[1] > 0 and self.bg.rect.top >= 0:
				self.bg.vel[1] = 0
				#self.g.dynamicLayer.vel[1] = 0
			if self.bg.vel[1] < 0 and self.bg.rect.bottom <= HEIGHT:
				self.bg.vel[1] = 0
				#self.g.dynamicLayer.vel[1] = 0
			
			currTime = pygame.time.get_ticks()
			if currTime - self.startTime >= 1000:
				self.timeLeft -= 1
				self.startTime = currTime

			if self.timeLeft == 0:
				self.state = "MENU"
				self.g.notifyStateChange()
				pygame.mixer.music.load(data.filepath(\
					os.path.join("sounds", "gameover.mid")))
				pygame.mixer.music.play(-1)
				self.timeLeft = 30

				
			self.g.updateGroups()
	
		if self.state == "MENU":
			pass
			
	def keyDown(self, key):
		
		if self.state == "PLAY":
			if key in DIRECTION_KEYS:
				self.keys.insert(0, key)
				self.adjustDirection()
			if key == K_ESCAPE:
				self.state = "PAUSE"
				self.g.notifyStateChange()
		elif self.state == "MENU":
			if key == K_DOWN:
				self.titleScreen.next()
			if key == K_UP:
				self.titleScreen.previous()
			if key == K_RETURN:
				self.titleScreen.updateState(self)
			if key == K_ESCAPE:
				sys.exit()
		elif self.state == "PAUSE":
			if key == K_DOWN:
				self.pauseScreen.next()
			if key == K_UP:
				self.pauseScreen.previous()
			if key == K_RETURN:
				self.pauseScreen.updateState(self)
			if key == K_ESCAPE:
				sys.exit()

	def keyUp(self, key):
		if self.state == "PLAY":
			if key in DIRECTION_KEYS:
				self.keys.remove(key)
				self.adjustDirection()

		elif self.state == "MENU":
			pass
	
	# On keydown and keyup events adjust list of directions to move.
	# Holding up, then right, then releasing right should continue up.
	def adjustDirection(self):
		if len(self.keys) > 0:
			self.animatePlayer = True
			if self.keys[0] == K_UP:
				self.bgVel = Vector2(0,1) * PLAYER_SPEED
				self.g.player.setPlayerDirection("Up")
			elif self.keys[0] == K_DOWN:
				self.bgVel = Vector2(0,-1) * PLAYER_SPEED
				self.g.player.setPlayerDirection("Down")
			elif self.keys[0] == K_LEFT:
				self.bgVel = Vector2(1,0) * PLAYER_SPEED
				self.g.player.setPlayerDirection("Left")
			elif self.keys[0] == K_RIGHT:
				self.bgVel = Vector2(-1,0) * PLAYER_SPEED
				self.g.player.setPlayerDirection("Right")
		else:
			self.bgVel = Vector2(0,0)
			self.animatePlayer = False


		# Update the velocity of all static sprites
		for obj in self.g.staticGroup:
			obj.setVel(self.bgVel)
		
		for obj in self.g.bgGroup:
			obj.setVel(self.bgVel)
		# Update the velocity of all dynamically moving sprites
		#for obj in self.g.dynamicGroup:
		#	obj.setVel(self.bgVel)

def main():
	Game()
